//
//  GameFieldViewController.m
//  OBJCM
//
//  Created by Pavel on 7/24/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "GameFieldViewController.h"

@interface GameFieldViewController ()
- (void)didInitializeGame;
- (void)setObjUIImgWithSelector:(UIImage *)uiImage;
- (void) spinLayer:(CALayer *)inLayer duration:(CFTimeInterval)inDuration direction:(int)direction hide:(BOOL)hideMark ifArray:(NSMutableArray *)arr;
- (void) didInitializeArrayHalfWithCards;
- (void) didDuplicateArrayHalfWithCards:(NSMutableArray *)halfArrCards;
- (NSMutableArray *) didShuffleArray:(NSMutableArray *)arr;
- (void) didResetParametresOfGameStatistic;
- (void) compareArrWithTwoCardObj: (CGFloat) withDelay;
- (void) deleteCards;
- (void) hideCards;
- (void) didCheckWinGame;
- (void) saveParametresForGameEnd:(BOOL)isGameOver isGameWin:(BOOL)isGameWin;
- (void) didDisableUserInteraction:(BOOL)YN;
- (void) initializeGameTimerToConcede;
- (void) didCheckLoseGame:(int) diff;
- (void) didContinueGameTimer;
- (void) didPlaySound:(NSString *)soundName;
- (void) fixDifficultyToLateGame;
@end

@implementation GameFieldViewController
@synthesize touchSenderCount;
@synthesize arrWithTwoCardObj;
@synthesize arrWithXY;
@synthesize countCards;
@synthesize cards;
@synthesize fieldCardsContainer;
@synthesize levelCount;
@synthesize timer;
@synthesize gamePoint;
@synthesize countLevelGame;
@synthesize globalTimer;
@synthesize gameToConcedeTimer;
@synthesize tick;
@synthesize maxTick;
@synthesize progressViewImg;
@synthesize soundSwitch;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

#pragma mark - View lifecycle
- (void) viewWillAppear:(BOOL)animated {
    [self initializeGameTimerToConcede];
    self.levelCount.text = [NSString stringWithFormat:@"Level %d", countLevelGame];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self didInitializeGame];
}

- (void) didInitializeGame {
    [self fixDifficultyToLateGame];
    fieldCardsContainer = [[UIView alloc] initWithFrame:CGRectMake(50, 100, 267, 295)];
    [self.view addSubview:fieldCardsContainer];
    [self didResetParametresOfGameStatistic];
    [self didInitializeArrayHalfWithCards];
}

- (void) initializeGameTimerToConcede {
    self.gameToConcedeTimer = [NSTimer scheduledTimerWithTimeInterval:1.0f target:self selector:@selector(didTickConcedeTimer) userInfo:nil repeats:YES];
}

- (void) didTickConcedeTimer {
    tick++;
    int dif = maxTick - tick;
    CGRect frame = [progressViewImg frame];
    frame.origin.x += 97/maxTick;
    frame.size.width -= 97/maxTick;
    if (frame.size.width > 0) {
        [progressViewImg setFrame:frame];
    }
    
    
    self.timer.text = [NSString stringWithFormat:@"%d",dif];
    [self didCheckLoseGame:dif];
}

- (void) didCheckLoseGame:(int) diff {
    if (diff==0) {
        if ([self.gameToConcedeTimer isValid]) {
            [self.gameToConcedeTimer invalidate];
        }
        [self saveParametresForGameEnd:YES isGameWin:NO];
    }
}

- (void) didResetParametresOfGameStatistic {
    tick = 0;
    self.gamePoint = 0;
    [progressViewImg setFrame:CGRectMake(214, 15, 97, 31)];
    self.arrWithXY = [[NSMutableArray alloc] initWithObjects:[NSNumber numberWithInt:0], [NSNumber numberWithInt:0], nil];
    self.arrWithTwoCardObj = nil;
    self.arrWithTwoCardObj = [[NSMutableArray alloc] init];
}

- (void) didInitializeArrayHalfWithCards {
    
    NSMutableArray *arr = [[NSMutableArray alloc] init];
    for (int i = 0; i < self.countCards; i++) {
        int randImgName = 1 + arc4random() % (20 - 1);
        NSMutableString *imageName = [NSMutableString stringWithFormat:@"%d.png",randImgName];
        UIImage *imageCard = [UIImage imageNamed:imageName];
        [self setObjUIImgWithSelector:imageCard];
        [arr addObject:imageCard];
    }
    [self didDuplicateArrayHalfWithCards:arr];
}

- (void) didDuplicateArrayHalfWithCards:(NSMutableArray *)halfArrCards {
    
    halfArrCards = [self didShuffleArray:halfArrCards];
    for (int i = 0; i < halfArrCards.count; i++) {
        UIImage *imageCard = [halfArrCards objectAtIndex:i];
        [self setObjUIImgWithSelector:imageCard];
    }
    
}

- (NSMutableArray *) didShuffleArray:(NSMutableArray *)arr {
    NSUInteger count = arr.count;
    for (NSUInteger i = 0; i < count; ++i) {
        int nElements = count - i;
        int n = (random() % nElements) + i;
        [arr exchangeObjectAtIndex:i withObjectAtIndex:n];
    }
    return arr;
}

- (void) setObjUIImgWithSelector:(UIImage *)uiImage {
    UIImage *defBackImg = [UIImage imageNamed:@"bgCard40.png"];
    UIImageView *defBackView = [[UIImageView alloc] initWithImage: defBackImg];
    UIImageView *imageView = [[UIImageView alloc] initWithImage: uiImage];
    
    [imageView setTag:1];
    imageView.userInteractionEnabled = YES;
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(didShowCard:)];
    
    NSInteger xx = [[self.arrWithXY objectAtIndex:0] integerValue];
    NSInteger yy = [[self.arrWithXY objectAtIndex:1] integerValue];
    imageView.frame = CGRectMake(0, 0, 40, 40);
    defBackView.frame = CGRectMake(0, 0, 40, 40);
    imageView.hidden = YES;
    UIView *cardOne = [[UIView alloc] initWithFrame:CGRectMake(xx, yy, 40, 40)];
    [cardOne addSubview:defBackView];
    [cardOne addSubview:imageView];
    if (xx == 200) {
        xx = 0;
        yy += 40;
    }else{
        xx += 40;
    }
    [cardOne addGestureRecognizer:tap];
    
    self.arrWithXY = nil;
    self.arrWithXY = [[NSMutableArray alloc] initWithObjects:[NSNumber numberWithInt:xx], [NSNumber numberWithInt:yy], nil];
    [fieldCardsContainer addSubview:cardOne];
    [self.cards addObject:cardOne];
    
}

- (void)didShowCard:(UITapGestureRecognizer *)recognizer  {
    CGFloat delay = 1.2;
    UIView *viewCard = (UIView *)recognizer.view;
    viewCard.userInteractionEnabled = NO;
    self.touchSenderCount ++;
    [self.arrWithTwoCardObj addObject:viewCard];
    UIImageView *imgOne = [viewCard viewWithTag:1];
    imgOne.hidden = NO;
    [self spinLayer:viewCard.layer duration:delay direction:1 hide:NO ifArray:nil];
    if (self.touchSenderCount == 2) {
        [self compareArrWithTwoCardObj:delay];
        [self didDisableUserInteraction:YES];
    }
    [self didPlaySound:@"hideitem.mp3"];
}

- (void) didDisableUserInteraction:(BOOL)YN {
    fieldCardsContainer.userInteractionEnabled = !YN;
}

- (void) compareArrWithTwoCardObj: (CGFloat) withDelay {
    
    self.globalTimer = [NSTimer scheduledTimerWithTimeInterval:1.2f target:self selector:@selector(performActionCompareImages) userInfo:nil repeats:NO];
}

- (void) performActionCompareImages {
    UIImageView *imgOne = [[self.arrWithTwoCardObj objectAtIndex:0] viewWithTag:1];
    UIImageView *imgTwo = [[self.arrWithTwoCardObj objectAtIndex:1] viewWithTag:1];
    if (imgOne.image == imgTwo.image) {
        [self deleteCards];
    }else{
        [self hideCards];
    }
    self.touchSenderCount = 0;
}

- (void) deleteCards {
    [self didPlaySound:@"deleteItem.mp3"];
    [[self.arrWithTwoCardObj objectAtIndex:0] removeFromSuperview];
    [[self.arrWithTwoCardObj objectAtIndex:1] removeFromSuperview];
    self.arrWithTwoCardObj = nil;
    self.arrWithTwoCardObj = [[NSMutableArray alloc] init];
    [self didDisableUserInteraction:NO];
    [self didCheckWinGame];
    
}

- (void) didCheckWinGame {
    self.gamePoint ++;
    if (self.gamePoint == self.countCards) {
        self.countCards ++;
        self.countLevelGame++;
        self.maxTick +=5;
        [self fixDifficultyToLateGame];
        [self saveParametresForGameEnd:NO isGameWin:YES];
        [self didResetGameStatistic:nil];
        
    }
}

- (void) saveParametresForGameEnd:(BOOL)isGameOver isGameWin:(BOOL)isGameWin {
    
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.soundSwitch = !soundSwitch;
    [[NSUserDefaults standardUserDefaults] setInteger:self.countLevelGame forKey:@"countLevelGame"];
    [[NSUserDefaults standardUserDefaults] setInteger:self.countCards forKey:@"countCards"];
    //[[NSUserDefaults standardUserDefaults] setInteger:self.time forKey:@"time"];
    
    MenuViewController *menuViewController = [[MenuViewController alloc] initWithNibName:@"MenuViewController" bundle:nil];
    if([self.gameToConcedeTimer isValid]){
        [self.gameToConcedeTimer invalidate];
    }
    menuViewController.isGameWin = isGameWin;
    menuViewController.isGameOver = isGameOver;
    menuViewController.isFirstInitialization = NO;
    menuViewController.countLevelGame = self.countLevelGame;
    menuViewController.maxTick = maxTick;
    menuViewController.countCards = countCards;
    
    
    [self.navigationController pushViewController:menuViewController animated:YES];
}

- (void) hideCards {
    UIView *imgOne = [self.arrWithTwoCardObj objectAtIndex:0];
    UIView *imgTwo = [self.arrWithTwoCardObj objectAtIndex:1];
    imgOne.userInteractionEnabled = YES;
    imgTwo.userInteractionEnabled = YES;
    [self spinLayer:nil duration:1 direction:1 hide:YES ifArray:self.arrWithTwoCardObj];
}

- (void) spinLayer:(CALayer *)inLayer duration:(CFTimeInterval)inDuration direction:(int)direction hide:(BOOL)hideMark ifArray:(NSMutableArray *)arr {
    CABasicAnimation* rotationAnimation;
    rotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.y"];
    rotationAnimation.toValue = [NSNumber numberWithFloat: M_PI * 2.0 * direction];
    rotationAnimation.duration = inDuration;
    rotationAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    if(arr){
        UIView *one = [self.arrWithTwoCardObj objectAtIndex:0];
        UIView *two = [self.arrWithTwoCardObj objectAtIndex:1];
        [one.layer addAnimation:rotationAnimation forKey:@"rotationAnimation"];
        [two.layer addAnimation:rotationAnimation forKey:@"rotationAnimation"];
    }else{
        [inLayer addAnimation:rotationAnimation forKey:@"rotationAnimation"];
    }
    if (hideMark) {
        self.globalTimer = [NSTimer scheduledTimerWithTimeInterval:1.2f target:self selector:@selector(performHideWithDelay) userInfo:nil repeats:NO];
    }
}

- (void) performHideWithDelay {
    [self didPlaySound:@"hideitem.mp3"];
    UIImageView *one = [[self.arrWithTwoCardObj objectAtIndex:0] viewWithTag:1];
    UIImageView *two = [[self.arrWithTwoCardObj objectAtIndex:1] viewWithTag:1];
    one.hidden = YES;
    two.hidden = YES;
    self.arrWithTwoCardObj = nil;
    self.arrWithTwoCardObj = [[NSMutableArray alloc] init];
    [self didDisableUserInteraction:NO];
}


- (void)viewDidUnload
{
    [super viewDidUnload];
    self.touchSenderCount = 0;
    [fieldCardsContainer removeFromSuperview];
    if([self.globalTimer isValid]){
        [self.globalTimer invalidate];
    }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (IBAction)didBackMenuController:(id)sender {
    
    [self performSelector:@selector(backMenuController)];
}

- (void) backMenuController {
    MenuViewController *menuViewController = [[MenuViewController alloc] initWithNibName:@"MenuViewController" bundle:nil];
    if([self.gameToConcedeTimer isValid]){
        [self.gameToConcedeTimer invalidate];
    }
    menuViewController.isGameOver = NO;
    menuViewController.isFirstInitialization = NO;
    menuViewController.countLevelGame = self.countLevelGame;
    menuViewController.maxTick = maxTick;
    menuViewController.countCards = countCards;
    [self.navigationController pushViewController:menuViewController animated:YES];
}

- (void) didContinueGameTimer {
    self.gameToConcedeTimer = [NSTimer scheduledTimerWithTimeInterval:1.0f target:self selector:@selector(didTickConcedeTimer) userInfo:nil repeats:YES];
}

- (void) didPlaySound:(NSString *)soundName {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    if (appDelegate.soundSwitch) {
        SystemSoundID volleyFile;
        NSString *volleyPath = [[NSBundle mainBundle] pathForResource:soundName ofType:nil];
        CFURLRef volleyURL = (__bridge_retained CFURLRef ) [NSURL fileURLWithPath:volleyPath];
        AudioServicesCreateSystemSoundID (volleyURL, &volleyFile);
        AudioServicesPlaySystemSound(volleyFile);
    }
}

- (void) fixDifficultyToLateGame {
    if (countCards>=21) {
        countCards=21;
    }
    if (maxTick >= 225) {
        maxTick -=5;
    }
}

- (IBAction)didResetGameStatistic:(id)sender {
    [self viewDidUnload];
    [self viewDidLoad];
}
@end
